import React, { createContext, useContext, useMemo, useState } from 'react';
import { OnomyHubConstants } from '@onomy/hub';
import { useBridgeForm, useFixedAmountInput, useOnomyHub } from '@onomy/react-hub';
import { FungibleAssetOnChain } from '@onomy/frontend-relay-types';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

function getAssetKey(asset: FungibleAssetOnChain) {
  return [asset.chain.name, asset.id].join('|');
}

type BridgeMap = {
  fromAsset: FungibleAssetOnChain;
  toAssets: Record<string, FungibleAssetOnChain>;
};

function useWalletBridgeFormState() {
  const { hub } = useOnomyHub();
  if (!hub) throw new Error('onomy hub not present');

  const bridgeAssetMap = useMemo(() => {
    return hub.bridges.reduce((res, bridge) => {
      const bridgeFromAsset = bridge.getFromAsset();
      const bridgeToAsset = bridge.getToAsset();
      const fromAssetKey = getAssetKey(bridgeFromAsset);
      const toAssetKey = getAssetKey(bridgeToAsset);

      // eslint-disable-next-line no-multi-assign
      const fromAssetRecord = (res[fromAssetKey] = res[fromAssetKey] || {
        fromAsset: bridgeFromAsset,
        toAssets: {},
      });

      fromAssetRecord.toAssets[toAssetKey] = bridgeToAsset;
      return res;
    }, {} as Record<string, BridgeMap>);
  }, [hub.bridges]);

  const fromAssets = useMemo(
    () => Object.values(bridgeAssetMap).map(v => v.fromAsset),
    [bridgeAssetMap]
  );

  const [fromAsset, setFromAsset] = useState(OnomyHubConstants.NOM_ASSET);

  const toAssets = useMemo(() => {
    const key = getAssetKey(fromAsset);
    return Object.values(bridgeAssetMap[key]?.toAssets ?? {});
  }, [fromAsset, bridgeAssetMap]);

  const [toAsset, setToAsset] = useState(OnomyHubConstants.ETH_NOM_ASSET);

  // TODO: reset toAsset on fromAsset change

  const form = useBridgeForm({
    fromAsset,
    toAsset,
  });

  const [amountDisplayValue, setAmountDisplayValue] = useFixedAmountInput({
    value: form.bridgeAmountFixed,
    onChangeValue: form.setBridgeAmountFixed,
    meta: form.fromAsset.meta,
  });

  return {
    ...form,
    fromAssets,
    toAssets,

    amountDisplayValue,

    setAmountDisplayValue,

    setFromAsset,
    setToAsset,
  };
}

export type WalletBridgeFormContextType = ReturnType<typeof useWalletBridgeFormState>;

const WalletBridgeFormContext = createContext<WalletBridgeFormContextType>(null as any);

export function useWalletBridgeForm() {
  const state = useContext(WalletBridgeFormContext);
  if (!state) throw new Error('ValidatorDetailContext missing');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_LOADING_RENDER_FN() {
  return <PlaceholderLoadingIndicator text="Loading bridge form" />;
}
// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function WalletBridgeFormProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
}: {
  children: JSX.Element | JSX.Element[];
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
}) {
  const state = useWalletBridgeFormState();

  if (state.loadingStatus.pending) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);
  return (
    <WalletBridgeFormContext.Provider value={state}>{children}</WalletBridgeFormContext.Provider>
  );
}
