import React, { createContext, useContext, useMemo } from 'react';
import { useProposals, useAssetBalance, useAddress, useProposalMinDeposit } from '@onomy/react-hub';
import { useAsyncStatuses } from '@onomy/react-utils';
import { useOnomyApp } from '../../wallet/providers/OnomyApp';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

function useGovernanceState() {
  const { onomyNomAsset: govAsset } = useOnomyApp(); // TODO: support alternate governance assets/chains
  const [myAddress, myAddressStatus] = useAddress(govAsset);
  const [allProposals, allProposalsStatus] = useProposals();
  const [myProposals, myProposalsStatus] = useProposals({ depositor: myAddress?.id });
  const [myVotedProposals, myVotedProposalsStatus] = useProposals({ voter: myAddress?.id });

  const govAssetBalance = useAssetBalance(govAsset);

  const [minDepositAmount, minDepositAmountStatus] = useProposalMinDeposit();

  const loadingStatus = useAsyncStatuses([
    myAddressStatus,
    allProposalsStatus,
    minDepositAmountStatus,
    myProposalsStatus,
    myVotedProposalsStatus,
  ]);

  const activeProposals = useMemo(() => {
    return allProposals.filter(proposal => {
      return proposal.status === 'PROPOSAL_STATUS_VOTING_PERIOD';
    });
  }, [allProposals]);

  const passedProposals = useMemo(() => {
    return allProposals.filter(proposal => {
      return proposal.status === 'PROPOSAL_STATUS_PASSED';
    });
  }, [allProposals]);

  const rejectedProposals = useMemo(() => {
    return allProposals.filter(proposal => {
      return (
        proposal.status === 'PROPOSAL_STATUS_REJECTED' ||
        proposal.status === 'PROPOSAL_STATUS_FAILED'
      );
    });
  }, [allProposals]);

  const failedProposals = useMemo(() => {
    return allProposals.filter(proposal => {
      return proposal.status === 'PROPOSAL_STATUS_FAILED';
    });
  }, [allProposals]);

  return {
    governanceRoute: '/governance',

    govAsset,
    govAssetBalance,
    myAddress,
    minDepositAmount,

    allProposals,
    myProposals,
    myVotedProposals,
    activeProposals,
    passedProposals,
    rejectedProposals,
    failedProposals,

    loadingStatus,
  };
}

export type GovernanceContextType = ReturnType<typeof useGovernanceState>;

const GovernanceContext = createContext<GovernanceContextType>(null as any);

export function useGovernance() {
  const state = useContext(GovernanceContext);
  if (!state) throw new Error('GovernanceContext missing');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_LOADING_RENDER_FN() {
  return <PlaceholderLoadingIndicator text="Loading governance" />;
}
// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function GovernanceProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
}: {
  children: JSX.Element | JSX.Element[];
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
}) {
  const state = useGovernanceState();

  if (state.loadingStatus.pending) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);
  return <GovernanceContext.Provider value={state}>{children}</GovernanceContext.Provider>;
}
