import React, { createContext, useContext } from 'react';
import { OnomyHubConstants } from '@onomy/hub';
import { useClaimRewardForm, useDelegation } from '@onomy/react-hub';
import { useAsyncStatuses } from '@onomy/react-utils';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

function useValidatorDetailState(validatorId: string) {
  const [delegation, delegationStatus] = useDelegation({
    chain: OnomyHubConstants.ONOMY_CHAIN, // TODO multiple chain support
    id: validatorId,
  });
  const claimRewards = useClaimRewardForm({ validatorAddressStr: validatorId });

  const loadingStatus = useAsyncStatuses([delegationStatus, claimRewards.loadingStatus]);

  const validatorDetailRoute = `/wallet/staking/${validatorId}`;

  return {
    delegation,
    claimRewards,

    loadingStatus,
    validatorDetailRoute,
  };
}

export type ValidatorDetailContextType = ReturnType<typeof useValidatorDetailState>;

const ValidatorDetailContext = createContext<ValidatorDetailContextType>(null as any);

export function useValidatorDetail() {
  const state = useContext(ValidatorDetailContext);
  if (!state) throw new Error('ValidatorDetailContext missing');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_LOADING_RENDER_FN() {
  return <PlaceholderLoadingIndicator text="Loading validator" />;
}
// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function ValidatorDetailProvider({
  children,
  getValidatorId,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
}: {
  children: JSX.Element | JSX.Element[];
  getValidatorId: () => string;
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
}) {
  const state = useValidatorDetailState(getValidatorId());

  if (state.loadingStatus.pending) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);
  if (!state.delegation) return renderLoading();
  return (
    <ValidatorDetailContext.Provider value={state}>{children}</ValidatorDetailContext.Provider>
  );
}
